from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricFieldUnits(Enum):
        """
            ElectricFieldUnits enumeration
        """
        
        VoltPerMeter = 'volt_per_meter'
        """
            
        """
        

class ElectricField(AbstractMeasure):
    """
    An electric field is a force field that surrounds electric charges that attracts or repels other electric charges.

    Args:
        value (float): The value.
        from_unit (ElectricFieldUnits): The ElectricField unit to create from, The default unit is VoltPerMeter
    """
    def __init__(self, value: float, from_unit: ElectricFieldUnits = ElectricFieldUnits.VoltPerMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__volts_per_meter = None
        

    def convert(self, unit: ElectricFieldUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: ElectricFieldUnits) -> float:
        value = self._value
        
        if from_unit == ElectricFieldUnits.VoltPerMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricFieldUnits) -> float:
        
        if to_unit == ElectricFieldUnits.VoltPerMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_volts_per_meter(volts_per_meter: float):
        """
        Create a new instance of ElectricField from a value in volts_per_meter.

        

        :param meters: The ElectricField value in volts_per_meter.
        :type volts_per_meter: float
        :return: A new instance of ElectricField.
        :rtype: ElectricField
        """
        return ElectricField(volts_per_meter, ElectricFieldUnits.VoltPerMeter)

    
    @property
    def volts_per_meter(self) -> float:
        """
        
        """
        if self.__volts_per_meter != None:
            return self.__volts_per_meter
        self.__volts_per_meter = self.__convert_from_base(ElectricFieldUnits.VoltPerMeter)
        return self.__volts_per_meter

    
    def to_string(self, unit: ElectricFieldUnits = ElectricFieldUnits.VoltPerMeter) -> str:
        """
        Format the ElectricField to string.
        Note! the default format for ElectricField is VoltPerMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == ElectricFieldUnits.VoltPerMeter:
            return f"""{self.volts_per_meter} V/m"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricFieldUnits = ElectricFieldUnits.VoltPerMeter) -> str:
        """
        Get ElectricField unit abbreviation.
        Note! the default abbreviation for ElectricField is VoltPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricFieldUnits.VoltPerMeter:
            return """V/m"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class LevelUnits(Enum):
        """
            LevelUnits enumeration
        """
        
        Decibel = 'decibel'
        """
            
        """
        
        Neper = 'neper'
        """
            
        """
        

class Level(AbstractMeasure):
    """
    Level is the logarithm of the ratio of a quantity Q to a reference value of that quantity, Q₀, expressed in dimensionless units.

    Args:
        value (float): The value.
        from_unit (LevelUnits): The Level unit to create from, The default unit is Decibel
    """
    def __init__(self, value: float, from_unit: LevelUnits = LevelUnits.Decibel):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__decibels = None
        
        self.__nepers = None
        

    def convert(self, unit: LevelUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: LevelUnits) -> float:
        value = self._value
        
        if from_unit == LevelUnits.Decibel:
            return (value)
        
        if from_unit == LevelUnits.Neper:
            return (0.115129254 * value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: LevelUnits) -> float:
        
        if to_unit == LevelUnits.Decibel:
            return (value)
        
        if to_unit == LevelUnits.Neper:
            return ((1 / 0.115129254) * value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_decibels(decibels: float):
        """
        Create a new instance of Level from a value in decibels.

        

        :param meters: The Level value in decibels.
        :type decibels: float
        :return: A new instance of Level.
        :rtype: Level
        """
        return Level(decibels, LevelUnits.Decibel)

    
    @staticmethod
    def from_nepers(nepers: float):
        """
        Create a new instance of Level from a value in nepers.

        

        :param meters: The Level value in nepers.
        :type nepers: float
        :return: A new instance of Level.
        :rtype: Level
        """
        return Level(nepers, LevelUnits.Neper)

    
    @property
    def decibels(self) -> float:
        """
        
        """
        if self.__decibels != None:
            return self.__decibels
        self.__decibels = self.__convert_from_base(LevelUnits.Decibel)
        return self.__decibels

    
    @property
    def nepers(self) -> float:
        """
        
        """
        if self.__nepers != None:
            return self.__nepers
        self.__nepers = self.__convert_from_base(LevelUnits.Neper)
        return self.__nepers

    
    def to_string(self, unit: LevelUnits = LevelUnits.Decibel) -> str:
        """
        Format the Level to string.
        Note! the default format for Level is Decibel.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == LevelUnits.Decibel:
            return f"""{self.decibels} dB"""
        
        if unit == LevelUnits.Neper:
            return f"""{self.nepers} Np"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: LevelUnits = LevelUnits.Decibel) -> str:
        """
        Get Level unit abbreviation.
        Note! the default abbreviation for Level is Decibel.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == LevelUnits.Decibel:
            return """dB"""
        
        if unit_abbreviation == LevelUnits.Neper:
            return """Np"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class PermittivityUnits(Enum):
        """
            PermittivityUnits enumeration
        """
        
        FaradPerMeter = 'farad_per_meter'
        """
            
        """
        

class Permittivity(AbstractMeasure):
    """
    In electromagnetism, permittivity is the measure of resistance that is encountered when forming an electric field in a particular medium.

    Args:
        value (float): The value.
        from_unit (PermittivityUnits): The Permittivity unit to create from, The default unit is FaradPerMeter
    """
    def __init__(self, value: float, from_unit: PermittivityUnits = PermittivityUnits.FaradPerMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__farads_per_meter = None
        

    def convert(self, unit: PermittivityUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: PermittivityUnits) -> float:
        value = self._value
        
        if from_unit == PermittivityUnits.FaradPerMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: PermittivityUnits) -> float:
        
        if to_unit == PermittivityUnits.FaradPerMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_farads_per_meter(farads_per_meter: float):
        """
        Create a new instance of Permittivity from a value in farads_per_meter.

        

        :param meters: The Permittivity value in farads_per_meter.
        :type farads_per_meter: float
        :return: A new instance of Permittivity.
        :rtype: Permittivity
        """
        return Permittivity(farads_per_meter, PermittivityUnits.FaradPerMeter)

    
    @property
    def farads_per_meter(self) -> float:
        """
        
        """
        if self.__farads_per_meter != None:
            return self.__farads_per_meter
        self.__farads_per_meter = self.__convert_from_base(PermittivityUnits.FaradPerMeter)
        return self.__farads_per_meter

    
    def to_string(self, unit: PermittivityUnits = PermittivityUnits.FaradPerMeter) -> str:
        """
        Format the Permittivity to string.
        Note! the default format for Permittivity is FaradPerMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == PermittivityUnits.FaradPerMeter:
            return f"""{self.farads_per_meter} F/m"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: PermittivityUnits = PermittivityUnits.FaradPerMeter) -> str:
        """
        Get Permittivity unit abbreviation.
        Note! the default abbreviation for Permittivity is FaradPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == PermittivityUnits.FaradPerMeter:
            return """F/m"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class MagnetizationUnits(Enum):
        """
            MagnetizationUnits enumeration
        """
        
        AmperePerMeter = 'ampere_per_meter'
        """
            
        """
        

class Magnetization(AbstractMeasure):
    """
    In classical electromagnetism, magnetization is the vector field that expresses the density of permanent or induced magnetic dipole moments in a magnetic material.

    Args:
        value (float): The value.
        from_unit (MagnetizationUnits): The Magnetization unit to create from, The default unit is AmperePerMeter
    """
    def __init__(self, value: float, from_unit: MagnetizationUnits = MagnetizationUnits.AmperePerMeter):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__amperes_per_meter = None
        

    def convert(self, unit: MagnetizationUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: MagnetizationUnits) -> float:
        value = self._value
        
        if from_unit == MagnetizationUnits.AmperePerMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: MagnetizationUnits) -> float:
        
        if to_unit == MagnetizationUnits.AmperePerMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_amperes_per_meter(amperes_per_meter: float):
        """
        Create a new instance of Magnetization from a value in amperes_per_meter.

        

        :param meters: The Magnetization value in amperes_per_meter.
        :type amperes_per_meter: float
        :return: A new instance of Magnetization.
        :rtype: Magnetization
        """
        return Magnetization(amperes_per_meter, MagnetizationUnits.AmperePerMeter)

    
    @property
    def amperes_per_meter(self) -> float:
        """
        
        """
        if self.__amperes_per_meter != None:
            return self.__amperes_per_meter
        self.__amperes_per_meter = self.__convert_from_base(MagnetizationUnits.AmperePerMeter)
        return self.__amperes_per_meter

    
    def to_string(self, unit: MagnetizationUnits = MagnetizationUnits.AmperePerMeter) -> str:
        """
        Format the Magnetization to string.
        Note! the default format for Magnetization is AmperePerMeter.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == MagnetizationUnits.AmperePerMeter:
            return f"""{self.amperes_per_meter} A/m"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: MagnetizationUnits = MagnetizationUnits.AmperePerMeter) -> str:
        """
        Get Magnetization unit abbreviation.
        Note! the default abbreviation for Magnetization is AmperePerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == MagnetizationUnits.AmperePerMeter:
            return """A/m"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RelativeHumidityUnits(Enum):
        """
            RelativeHumidityUnits enumeration
        """
        
        Percent = 'percent'
        """
            
        """
        

class RelativeHumidity(AbstractMeasure):
    """
    Relative humidity is a ratio of the actual water vapor present in the air to the maximum water vapor in the air at the given temperature.

    Args:
        value (float): The value.
        from_unit (RelativeHumidityUnits): The RelativeHumidity unit to create from, The default unit is Percent
    """
    def __init__(self, value: float, from_unit: RelativeHumidityUnits = RelativeHumidityUnits.Percent):
        if math.isnan(value):
            raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__percent = None
        

    def convert(self, unit: RelativeHumidityUnits) -> float:
        return self.__convert_from_base(unit)

    def __convert_from_base(self, from_unit: RelativeHumidityUnits) -> float:
        value = self._value
        
        if from_unit == RelativeHumidityUnits.Percent:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RelativeHumidityUnits) -> float:
        
        if to_unit == RelativeHumidityUnits.Percent:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_percent(percent: float):
        """
        Create a new instance of RelativeHumidity from a value in percent.

        

        :param meters: The RelativeHumidity value in percent.
        :type percent: float
        :return: A new instance of RelativeHumidity.
        :rtype: RelativeHumidity
        """
        return RelativeHumidity(percent, RelativeHumidityUnits.Percent)

    
    @property
    def percent(self) -> float:
        """
        
        """
        if self.__percent != None:
            return self.__percent
        self.__percent = self.__convert_from_base(RelativeHumidityUnits.Percent)
        return self.__percent

    
    def to_string(self, unit: RelativeHumidityUnits = RelativeHumidityUnits.Percent) -> str:
        """
        Format the RelativeHumidity to string.
        Note! the default format for RelativeHumidity is Percent.
        To specify the unit format set the 'unit' parameter.
        """
        
        if unit == RelativeHumidityUnits.Percent:
            return f"""{self.percent} %RH"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RelativeHumidityUnits = RelativeHumidityUnits.Percent) -> str:
        """
        Get RelativeHumidity unit abbreviation.
        Note! the default abbreviation for RelativeHumidity is Percent.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RelativeHumidityUnits.Percent:
            return """%RH"""
        